.timestamp {
  font-size: 0.8em;
  padding-left: 0.5in;
  color: #aaa;
}

#new_person {
  width:35em;
  margin:auto;
}

/* Pager component CSS */

.pager_window_info {
  text-align:center;
  margin-top:0; 
  margin-bottom: .5rem;
}

.pager_page {
  margin: 0.5em;
}

.pager_current_page {
  font-weight: bold;
  text-decoration: underline;
}

/* End Pager component CSS */
