{
  en => {
    errors => {
      format => '{{attribute}} {{message}}',
      messages => {
        invalid => {
          one => "Is Invalid",
          other => "Are Invalid",
          _default => "Is Invalid",
        },
      },
    }
  },
};
